@@ -46,6 +46,8 @@
 #include "src/nl80211cmd.h"
 #include "src/nl80211util.h"
 #include "src/util.h"
+#include "src/p2putil.h"
+#include "src/mpdu.h"
 #include "src/scan.h"
 
 #define SCAN_MAX_INTERVAL 320
@@ -75,6 +77,8 @@ struct scan_request {
 	scan_destroy_func_t destroy;
 	bool passive:1; /* Active or Passive scan? */
 	struct l_queue *cmds;
+	/* The time the current scan was started. Reported in TRIGGER_SCAN */
+	uint64_t start_time_tsf;
 };
 
 struct scan_context {
@@ -396,6 +400,19 @@ static struct l_genl_msg *scan_build_cmd(struct scan_context *sc,
 		l_free(scan_rates);
 	}
 
+	if (wiphy_has_ext_feature(sc->wiphy,
+					NL80211_EXT_FEATURE_SET_SCAN_DWELL)) {
+		if (params->duration)
+			l_genl_msg_append_attr(msg,
+					NL80211_ATTR_MEASUREMENT_DURATION,
+					2, &params->duration);
+
+		if (params->duration_mandatory)
+			l_genl_msg_append_attr(msg,
+				NL80211_ATTR_MEASUREMENT_DURATION_MANDATORY,
+				0, NULL);
+	}
+
 done:
 	return msg;
 }
@@ -556,6 +573,16 @@ uint32_t scan_passive(uint64_t wdev_id, struct scan_freq_set *freqs,
 							userdata, destroy);
 }
 
+uint32_t scan_passive_full(uint64_t wdev_id,
+			const struct scan_parameters *params,
+			scan_trigger_func_t trigger,
+			scan_notify_func_t notify, void *userdata,
+			scan_destroy_func_t destroy)
+{
+	return scan_common(wdev_id, true, params, trigger,
+				notify, userdata, destroy);
+}
+
 uint32_t scan_active(uint64_t wdev_id, uint8_t *extra_ie, size_t extra_ie_size,
 			scan_trigger_func_t trigger,
 			scan_notify_func_t notify, void *userdata,
@@ -755,6 +782,25 @@ bool scan_periodic_stop(uint64_t wdev_id)
 	return true;
 }
 
+uint64_t scan_get_triggered_time(uint64_t wdev_id, uint32_t id)
+{
+	struct scan_context *sc;
+	struct scan_request *sr;
+
+	sc = l_queue_find(scan_contexts, scan_context_match, &wdev_id);
+	if (!sc)
+		return 0;
+
+	if (!sc->triggered)
+		return 0;
+
+	sr = l_queue_find(sc->requests, scan_request_match, L_UINT_TO_PTR(id));
+	if (!sr)
+		return 0;
+
+	return sr->start_time_tsf;
+}
+
 static void scan_periodic_timeout(struct l_timeout *timeout, void *user_data)
 {
 	struct scan_context *sc = user_data;
@@ -1000,6 +1046,65 @@ static bool scan_parse_bss_information_elements(struct scan_bss *bss,
 		}
 	}
 
+	bss->wsc = ie_tlv_extract_wsc_payload(data, len, &bss->wsc_size);
+
+	switch (bss->source_frame) {
+	case SCAN_BSS_PROBE_RESP:
+		bss->p2p_probe_resp_info = l_new(struct p2p_probe_resp, 1);
+
+		if (p2p_parse_probe_resp(data, len, bss->p2p_probe_resp_info) ==
+				0)
+			break;
+
+		l_free(bss->p2p_probe_resp_info);
+		bss->p2p_probe_resp_info = NULL;
+		break;
+	case SCAN_BSS_PROBE_REQ:
+		bss->p2p_probe_req_info = l_new(struct p2p_probe_req, 1);
+
+		if (p2p_parse_probe_req(data, len, bss->p2p_probe_req_info) ==
+				0)
+			break;
+
+		l_free(bss->p2p_probe_req_info);
+		bss->p2p_probe_req_info = NULL;
+		break;
+	case SCAN_BSS_BEACON:
+	{
+		/*
+		 * Beacon and Probe Response P2P IE subelement formats are
+		 * mutually incompatible and can help us distinguish one frame
+		 * subtype from the other if the driver is not exposing enough
+		 * information.  As a result of trusting the frame contents on
+		 * this, no critical code should depend on the
+		 * bss->source_frame information being right.
+		 */
+		struct p2p_beacon info;
+		int r;
+
+		r = p2p_parse_beacon(data, len, &info);
+		if (r == 0) {
+			bss->p2p_beacon_info = l_memdup(&info, sizeof(info));
+			break;
+		}
+
+		if (r == -ENOENT)
+			break;
+
+		bss->p2p_probe_resp_info = l_new(struct p2p_probe_resp, 1);
+
+		if (p2p_parse_probe_resp(data, len, bss->p2p_probe_resp_info) ==
+				0) {
+			bss->source_frame = SCAN_BSS_PROBE_RESP;
+			break;
+		}
+
+		l_free(bss->p2p_probe_resp_info);
+		bss->p2p_probe_resp_info = NULL;
+		break;
+	}
+	}
+
 	return have_ssid;
 }
 
@@ -1008,9 +1113,14 @@ static struct scan_bss *scan_parse_attr_bss(struct l_genl_attr *attr)
 	uint16_t type, len;
 	const void *data;
 	struct scan_bss *bss;
+	const uint8_t *ies = NULL;
+	size_t ies_len;
+	const uint8_t *beacon_ies = NULL;
+	size_t beacon_ies_len;
 
 	bss = l_new(struct scan_bss, 1);
 	bss->utilization = 127;
+	bss->source_frame = SCAN_BSS_BEACON;
 
 	while (l_genl_attr_next(attr, &type, &len, &data)) {
 		switch (type) {
@@ -1039,19 +1149,39 @@ static struct scan_bss *scan_parse_attr_bss(struct l_genl_attr *attr)
 			bss->signal_strength = *((int32_t *) data);
 			break;
 		case NL80211_BSS_INFORMATION_ELEMENTS:
-			if (!scan_parse_bss_information_elements(bss,
-								data, len))
+			ies = data;
+			ies_len = len;
+			break;
+		case NL80211_BSS_PARENT_TSF:
+			if (len != sizeof(uint64_t))
 				goto fail;
 
-			bss->wsc = ie_tlv_extract_wsc_payload(data, len,
-								&bss->wsc_size);
-			bss->p2p = ie_tlv_extract_p2p_payload(data, len,
-								&bss->p2p_size);
-
+			bss->parent_tsf = l_get_u64(data);
+			break;
+		case NL80211_BSS_PRESP_DATA:
+			bss->source_frame = SCAN_BSS_PROBE_RESP;
+			break;
+		case NL80211_BSS_BEACON_IES:
+			beacon_ies = data;
+			beacon_ies_len = len;
 			break;
 		}
 	}
 
+	/*
+	 * Try our best at deciding whether the IEs come from a Probe
+	 * Response based on the hints explained in nl80211.h
+	 * (enum nl80211_bss).
+	 */
+	if (bss->source_frame == SCAN_BSS_BEACON && ies && (
+				!beacon_ies ||
+				ies_len != beacon_ies_len ||
+				memcmp(ies, beacon_ies, ies_len)))
+		bss->source_frame = SCAN_BSS_PROBE_RESP;
+
+	if (ies && !scan_parse_bss_information_elements(bss, ies, ies_len))
+		goto fail;
+
 	return bss;
 
 fail:
@@ -1205,15 +1335,65 @@ static void scan_bss_compute_rank(struct scan_bss *bss)
 		bss->rank = irank;
 }
 
+struct scan_bss *scan_bss_new_from_probe_req(const struct mmpdu_header *mpdu,
+						const uint8_t *body,
+						size_t body_len,
+						uint32_t frequency, int rssi)
+
+{
+	struct scan_bss *bss;
+
+	bss = l_new(struct scan_bss, 1);
+	memcpy(bss->addr, mpdu->address_2, 6);
+	bss->utilization = 127;
+	bss->source_frame = SCAN_BSS_PROBE_REQ;
+	bss->frequency = frequency;
+	bss->signal_strength = rssi;
+
+	if (!scan_parse_bss_information_elements(bss, body, body_len))
+		goto fail;
+
+	scan_bss_compute_rank(bss);
+	return bss;
+
+fail:
+	scan_bss_free(bss);
+	return NULL;
+}
+
 void scan_bss_free(struct scan_bss *bss)
 {
 	l_free(bss->ext_supp_rates_ie);
 	l_free(bss->rsne);
 	l_free(bss->wpa);
 	l_free(bss->wsc);
-	l_free(bss->p2p);
 	l_free(bss->osen);
 	l_free(bss->rc_ie);
+
+	switch (bss->source_frame) {
+	case SCAN_BSS_PROBE_RESP:
+		if (!bss->p2p_probe_resp_info)
+			break;
+
+		p2p_clear_probe_resp(bss->p2p_probe_resp_info);
+		l_free(bss->p2p_probe_resp_info);
+		break;
+	case SCAN_BSS_PROBE_REQ:
+		if (!bss->p2p_probe_req_info)
+			break;
+
+		p2p_clear_probe_req(bss->p2p_probe_req_info);
+		l_free(bss->p2p_probe_req_info);
+		break;
+	case SCAN_BSS_BEACON:
+		if (!bss->p2p_beacon_info)
+			break;
+
+		p2p_clear_beacon(bss->p2p_beacon_info);
+		l_free(bss->p2p_beacon_info);
+		break;
+	}
+
 	l_free(bss);
 }
 
@@ -1410,6 +1590,8 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 	uint32_t wiphy_id;
 	struct scan_context *sc;
 	bool active_scan = false;
+	uint64_t start_time_tsf = 0;
+	struct scan_request *sr;
 
 	cmd = l_genl_msg_get_command(msg);
 
@@ -1432,15 +1614,22 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 		case NL80211_ATTR_SCAN_SSIDS:
 			active_scan = true;
 			break;
+		case NL80211_ATTR_SCAN_START_TIME_TSF:
+			if (len != sizeof(uint64_t))
+				return;
+
+			start_time_tsf = l_get_u64(data);
+			break;
 		}
 	}
 
+	sr = l_queue_peek_head(sc->requests);
+
 	switch (cmd) {
 	case NL80211_CMD_NEW_SCAN_RESULTS:
 	{
 		struct l_genl_msg *scan_msg;
 		struct scan_results *results;
-		struct scan_request *sr = l_queue_peek_head(sc->requests);
 		bool send_next = false;
 		bool get_results = false;
 
@@ -1514,12 +1703,11 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 		else
 			sc->state = SCAN_STATE_PASSIVE;
 
+		sr->start_time_tsf = start_time_tsf;
+
 		break;
 
 	case NL80211_CMD_SCAN_ABORTED:
-	{
-		struct scan_request *sr = l_queue_peek_head(sc->requests);
-
 		if (sc->state == SCAN_STATE_NOT_RUNNING)
 			break;
 
@@ -1528,8 +1716,7 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 		if (sc->triggered) {
 			sc->triggered = false;
 
-			scan_finished(sc, -ECANCELED, NULL,
-					l_queue_peek_head(sc->requests));
+			scan_finished(sc, -ECANCELED, NULL, sr);
 		} else if (sr && !sc->start_cmd_id && !sc->get_scan_cmd_id) {
 			/*
 			 * If this was an external scan that got aborted
@@ -1543,7 +1730,6 @@ static void scan_notify(struct l_genl_msg *msg, void *user_data)
 
 		break;
 	}
-	}
 }
 
 uint8_t scan_freq_to_channel(uint32_t freq, enum scan_band *out_band)
